#menubar { 
    padding-left: 15px;
}

.chart {
    position: relative;
    margin-bottom: 20px;
}

.grid .tick {
    stroke: darkgrey;
    stroke-dasharray: 1, 2;
    opacity: 0.9;
}
.grid path {
    stroke-width: 0;
}

.line_over { 
    stroke: darkgrey;
    stroke-width: 1.5px;
    stroke-dasharray: 1, 2;
    display: none;
}

.axis { 
    font-size: 0.7em;
}

.axis path,
.axis line {
    fill: none;
    stroke: #000;
    shape-rendering: crispEdges;
}

.area, .area2 { 
    fill-opacity: .6;
}

.brush .extent {
    stroke: #fff;
    fill-opacity: .125;
    shape-rendering: crispEdges;
}

.legend rect {
    fill: none;
    stroke: black;
    opacity: 0;
}

.tip {
    position: absolute;
    z-index: 20;
    top: 60px;
    display: none;
    font-size: 0.9em;
    padding: 5px;
    background: rgba(0, 0, 0, 0.6);
    color: #fff;
    border-radius: 5px;
}

.tip p { 
    margin: 0;
}

.tracker { 
    display: none;
}